@@ -418,6 +418,9 @@ class Schema (object):
             raise ValueError('Schema %s does not appear to belong to model.' % (self,))
         self.catalog.delete(self.uri_path).raise_for_status()
         del self.model.schemas[self.name]
+        for table in self.tables.values():
+            for fkey in table.foreign_keys:
+                fkey._cleanup()
 
     @object_annotation(tag.display)
     def display(self): pass
@@ -970,6 +973,8 @@ class Table (object):
             raise ValueError('Table %s does not appear to belong to schema %s.' % (self, self.schema))
         self.catalog.delete(self.uri_path).raise_for_status()
         del self.schema.tables[self.name]
+        for fkey in self.foreign_keys:
+            fkey._cleanup()
 
     def key_by_columns(self, unique_columns, raise_nomatch=True):
         """Return key from self.keys with matching unique columns.
@@ -1771,7 +1776,16 @@ class ForeignKey (object):
             raise ValueError('Foreign key %s does not appear to belong to table %s.' % (self, self.table))
         self.catalog.delete(self.uri_path).raise_for_status()
         del self.table.foreign_keys[self.name]
+        self._cleanup()
+
+    def _cleanup(self):
+        """Cleanup references in the local model following drop from remote database.
+        """
         del self.pk_table.referenced_by[self.name]
+        if self.constraint_schema:
+            del self.constraint_schema._fkeys[self.constraint_name]
+        else:
+            del self.table.schema.model._pseudo_fkeys[self.constraint_name]
 
     @object_annotation(tag.foreign_key)
     def foreign_key(self): pass
