@@ -3,7 +3,7 @@ import datetime
 import requests
 import logging
 from . import format_exception, NotModified, DEFAULT_HEADERS, DEFAULT_CHUNK_SIZE, MAX_CHUNK_SIZE, urlquote, Megabyte, \
-    get_transfer_summary, MaxRetryError
+    get_transfer_summary
 from .deriva_binding import DerivaBinding
 from .utils import hash_utils as hu, mime_utils as mu
 
@@ -154,7 +154,8 @@ class HatracStore(DerivaBinding):
                 sha256=None,
                 parents=True,
                 content_type=None,
-                content_disposition=None):
+                content_disposition=None,
+                allow_versioning=True):
         """Idempotent upload of object, returning object location URI.
 
            Arguments:
@@ -166,6 +167,7 @@ class HatracStore(DerivaBinding):
               parents: automatically create parent namespace(s) if missing
               content_type: the content-type of the object (optional)
               content_disposition: the preferred content-disposition of the object (optional)
+              allow_versioning: reject with NotModified if content already exists (optional)
            Automatically computes and sends Content-MD5 if no digests provided.
 
            If an object-version already exists under the same name
@@ -189,13 +191,18 @@ class HatracStore(DerivaBinding):
 
         try:
             r = self.head(path)
-            if r.status_code == 200 and \
-                    (md5 and r.headers.get('Content-MD5') == md5 or
-                     sha256 and r.headers.get('Content-SHA256') == sha256):
-                # object already has same content so skip upload
-                f.close()
-                return r.headers.get('Content-Location')
+            if r.status_code == 200:
+                if (md5 and r.headers.get('Content-MD5') == md5 or
+                        sha256 and r.headers.get('Content-SHA256') == sha256):
+                    # object already has same content so skip upload
+                    f.close()
+                    return r.headers.get('Content-Location')
+                elif not allow_versioning:
+                    raise NotModified("The data cannot be uploaded because content already exists for this object "
+                                      "and multiple versions are not allowed.")
         except requests.HTTPError as e:
+            if e.response.status_code != 404:
+                logging.debug("HEAD request failed: %s" % format_exception(e))
             pass
 
         # TODO: verify incoming hashes if supplied?
@@ -238,7 +245,8 @@ class HatracStore(DerivaBinding):
                 chunk_size=DEFAULT_CHUNK_SIZE,
                 create_parents=True,
                 allow_versioning=True,
-                callback=None):
+                callback=None,
+                cancel_job_on_error=True):
         """
         :param path:
         :param file_path:
@@ -252,6 +260,7 @@ class HatracStore(DerivaBinding):
         :param create_parents:
         :param allow_versioning:
         :param callback:
+        :param cancel_job_on_error:
         :return:
         """
         self.check_path(path)
@@ -264,47 +273,54 @@ class HatracStore(DerivaBinding):
                                 sha256,
                                 content_type=content_type,
                                 content_disposition=content_disposition,
-                                parents=create_parents)
+                                parents=create_parents,
+                                allow_versioning=allow_versioning)
 
         if not (md5 or sha256):
             md5 = hu.compute_file_hashes(file_path, hashes=['md5'])['md5'][1]
 
         try:
             r = self.head(path)
-            if r.status_code == 200 and \
-                    (md5 and r.headers.get('Content-MD5') == md5 or
-                     sha256 and r.headers.get('Content-SHA256') == sha256):
-                # object already has same content so skip upload
-                return r.headers.get('Content-Location')
-            elif not allow_versioning:
-                raise NotModified("The file [%s] cannot be uploaded because content already exists for this object "
-                                  "and multiple versions are not allowed." % file_path)
+            if r.status_code == 200:
+                if (md5 and r.headers.get('Content-MD5') == md5 or
+                        sha256 and r.headers.get('Content-SHA256') == sha256):
+                    # object already has same content so skip upload
+                    return r.headers.get('Content-Location')
+                elif not allow_versioning:
+                    raise NotModified("The file [%s] cannot be uploaded because content already exists for this object "
+                                      "and multiple versions are not allowed." % file_path)
         except requests.HTTPError as e:
             if e.response.status_code != 404:
                 logging.debug("HEAD request failed: %s" % format_exception(e))
             pass
 
+        job_id = self.create_upload_job(path,
+                                        file_path,
+                                        md5,
+                                        sha256,
+                                        content_type=content_type,
+                                        content_disposition=content_disposition,
+                                        create_parents=create_parents)
         try:
-            job_id = self.create_upload_job(path,
-                                            file_path,
-                                            md5,
-                                            sha256,
-                                            content_type=content_type,
-                                            content_disposition=content_disposition,
-                                            create_parents=create_parents)
-            self.put_obj_chunked(path, file_path, job_id, chunk_size, callback)
+            self.put_obj_chunked(path,
+                                 file_path,
+                                 job_id,
+                                 chunk_size=chunk_size,
+                                 callback=callback,
+                                 cancel_job_on_error=cancel_job_on_error)
             return self.finalize_upload_job(path, job_id)
-        except (requests.Timeout, MaxRetryError) as e:
+        except (requests.Timeout, requests.ConnectionError, requests.exceptions.RetryError) as e:
             raise HatracJobTimeout(e)
 
-    def put_obj_chunked(self, path, file_path, job_id, chunk_size=DEFAULT_CHUNK_SIZE, callback=None, start_chunk=0):
+    def put_obj_chunked(self, path, file_path, job_id,
+                        chunk_size=DEFAULT_CHUNK_SIZE, callback=None, start_chunk=0, cancel_job_on_error=True):
         self.check_path(path)
         if chunk_size > MAX_CHUNK_SIZE:
             chunk_size = MAX_CHUNK_SIZE
         job_info = self.get_upload_job(path, job_id).json()
         try:
             file_size = os.path.getsize(file_path)
-            chunks = file_size / chunk_size
+            chunks = file_size // chunk_size
             if file_size % chunk_size:
                 chunks += 1
             with open(file_path, 'rb') as f:
@@ -341,13 +357,12 @@ class HatracStore(DerivaBinding):
                 logging.info("File [%s] upload successful. %s" % (file_path, summary))
                 if callback:
                     callback(summary=summary, file_path=file_path)
-        except (HatracJobAborted, HatracJobPaused):
-            raise
         except:
-            try:
-                self.cancel_upload_job(path, job_id)
-            except:
-                pass
+            if cancel_job_on_error:
+                try:
+                    self.cancel_upload_job(path, job_id)
+                except:
+                    pass
             raise
 
     def create_upload_job(self,
