@@ -12,6 +12,7 @@ from collections import OrderedDict, namedtuple
 from deriva.core import ErmrestCatalog, HatracStore, HatracJobAborted, HatracJobPaused, \
     HatracJobTimeout, urlquote, urlparse, stob, format_exception, get_credential, read_config, write_config, \
     copy_config, resource_path, make_dirs, lock_file, DEFAULT_CHUNK_SIZE, IS_PY2, __version__ as VERSION
+from deriva.core import DEFAULT_SESSION_CONFIG
 from deriva.core.utils import hash_utils as hu, mime_utils as mu, version_utils as vu
 from deriva.transfer.upload import *
 from deriva.transfer.upload.processors import find_processor
@@ -111,8 +112,7 @@ class DerivaUpload(object):
         host = self.server.get('host', '')
         self.server_url = protocol + "://" + host
         catalog_id = self.server.get("catalog_id", "1")
-        session_config = self.server.get('session')
-
+        session_config = self.server.get('session', DEFAULT_SESSION_CONFIG.copy())
         # overriden credential initialization
         if self.override_credential_file:
             self.credentials = get_credential(host, self.override_config_file)
@@ -395,12 +395,15 @@ class DerivaUpload(object):
         return result
 
     def validateFile(self, root, path, name):
-        file_path = os.path.normpath(os.path.join(path, name))
+        if self.config.get("relative_path_validation", False):
+            file_path = os.path.normpath(os.path.join(os.path.relpath(path, root), name))
+        else:
+            file_path = os.path.normpath(os.path.join(path, name))
         asset_group, asset_mapping, groupdict = self.getAssetMapping(file_path)
         if not asset_mapping:
             return None
 
-        return asset_group, asset_mapping, groupdict, file_path
+        return asset_group, asset_mapping, groupdict, os.path.abspath(os.path.normpath(os.path.join(path, name)))
 
     def scanDirectory(self, root, abort_on_invalid_input=False, purge_state=False):
         """
@@ -816,7 +819,8 @@ class DerivaUpload(object):
                                            file_path,
                                            job_id,
                                            callback=callback,
-                                           start_chunk=transfer_state["completed"])
+                                           start_chunk=transfer_state["completed"],
+                                           cancel_job_on_error=False)
             return self.store.finalize_upload_job(path, job_id)
         else:
             logger.info("Uploading file: [%s] to host %s. Please wait..." % (
@@ -830,7 +834,8 @@ class DerivaUpload(object):
                                       chunked=chunked,
                                       create_parents=create_parents,
                                       allow_versioning=allow_versioning,
-                                      callback=callback)
+                                      callback=callback,
+                                      cancel_job_on_error=False)
 
     def _get_catalog_table_columns(self, table):
         table_columns = set()
@@ -847,13 +852,19 @@ class DerivaUpload(object):
         return set(row.keys()) - self._get_catalog_table_columns(table)
 
     def _validate_row_key_constraints(self, catalog_table, row):
+        logger.debug("Validating row key constraints for %s: %s" % (catalog_table, row))
         schema_name, table_name = self.catalog.splitQualifiedCatalogName(catalog_table)
         schema = self.catalog_model.schemas.get(schema_name)
         table = schema.tables.get(table_name)
         non_null_correlations = {cname for cname, cval in row.items() if cval is not None}
         for key in table.keys:
-            if set(key.unique_columns).issubset(non_null_correlations):
+            if set(key.unique_columns.elements).issubset(non_null_correlations):
+                logger.debug("%s is a subset of non-null correlations %s" %
+                             (set(key.unique_columns.elements), non_null_correlations))
                 return True  # it is safe
+            else:
+                logger.debug("%s is not a subset of non-null correlations %s" %
+                             (set(key.unique_columns.elements), non_null_correlations))
         return False  # it is not safe
 
     def _get_catalog_default_columns(self, row, table, exclude=None, quote_url=True):
